package dk.au.daimi.rabbithunt.main;

import java.util.ArrayList;
import java.util.Collections;
import dk.au.daimi.rabbithunt.control.Controller;

/**
 * Executes a number of "rabbit hunts", in which a fox moves around a field
 * containing bushes, looking for a rabbit, and prints out the percentage of
 * times that the rabbit escapes.
 * 
 * Update to remove upper and lower five result. Sometimes the results are out
 * of proportion
 * 
 * @author David Matuszek
 * @author Anders Bach Nielsen - abachn@daimi.au.dk
 * @version April 07, 2006
 */
public class Grader
{
	public static final int NUMBER_OF_TRIALS = 1010;
	//public static final int NUMBER_OF_TRIALS = 3030;
	// class variables
	private static Controller model;
	private static int numberOfRows;
	private static int numberOfColumns;
	public static int numberOfFoxes = 2;

	/**
	 * Main class for starting rabbit hunts and counting games.
	 */
	public static void main(String[] args)
	{
		numberOfRows = numberOfColumns = 20;
		model = new Controller(numberOfRows, numberOfColumns, numberOfFoxes);
		countGames();
	}
	/**
	 * Runs NUMBER_OF_TRIALS rabbit hunts, and prints out the results as a
	 * percentage of times the rabbit escapes.
	 */
	private static void countGames()
	{
		ArrayList<Integer> steps = new ArrayList<Integer>(NUMBER_OF_TRIALS);
		int minSteps = Integer.MAX_VALUE;
		int maxSteps = 0;
		int count = 0, count2 = 0;
		for (int i = 0; i < NUMBER_OF_TRIALS; i++)
		{
			model.reset();
			while (!model.gameIsOver())
			{
				model.allowSingleMove();
				if (model.stepsTaken() == 3000)
				{
					count++;
					System.out.println("MERE END 3000 - " + count + " (tur: " + i + ")");
				}
				if (model.stepsTaken() == 5000)
				{
					count2++;
					System.out.println("MERE END 5000 - " + count2);
				}
				if (model.stepsTaken() == 50000)
					System.out.println("MERE END 50000 ");
				if (model.stepsTaken() == 100000)
				{
					System.out.println("MERE END 100000");
					System.out.println(model.oldRandomSeed);
					break; //uendelig lkke
				}
			}
			steps.add(model.stepsTaken());
		}
		Collections.sort(steps);
		// for (int i = 0; i < 15; i++)
		for (int i = 0; i < 5; i++)
		{
			steps.remove(NUMBER_OF_TRIALS - 1 - i);
		}
		// for (int i = 0; i < 15; i++)
		for (int i = 0; i < 5; i++)
		{
			steps.remove(0);
		}
		minSteps = steps.get(0);
		maxSteps = steps.get(NUMBER_OF_TRIALS - 11);
		// maxSteps = steps.get(NUMBER_OF_TRIALS - 31);
		int totalSteps = 0;
		// for (int i = 0; i < NUMBER_OF_TRIALS - 30; i++)
		for (int i = 0; i < NUMBER_OF_TRIALS - 10; i++)
		{
			totalSteps += steps.get(i);
		}
		System.out.printf("Rabbit survives for %.2f steps %n  on average "
				+ "(%d to %d steps) with %d foxes%n", (double) totalSteps
				/ (NUMBER_OF_TRIALS - 10), minSteps, maxSteps, numberOfFoxes);
	}
}
