package dk.au.daimi.rabbithunt.model;

import dk.au.daimi.rabbithunt.control.Controller;
import dk.au.daimi.rabbithunt.model.world.Direction;
import dk.au.daimi.rabbithunt.model.world.World;

/**
 * Represents an animal.
 *
 * @author David Matuszek
 * @author Martin Geisler
 * @author Mikkel Krigrd
 * @version August 16, 2005
 */
public class Animal extends Piece {

    // a very brainless brain
    private static class DefaultBrain extends Brain {

        public Direction decideMove() {
            return Direction.NONE;
        }

    }

    private World world;
    private Brain brain;

    /**
     * Constructor for objects of class Animal.
     *
     * @param model the model
     * (needed to create the animal's view of the world)
     */
    public Animal(Controller model) {
        super(null);
        world = new World(this, model);
        // Let's at least have this brain instead of just null. Now it
        // doesn't crash unless some really evil person sets the brain
        // to null.
        brain = new DefaultBrain();
    }

    /**
     * Decides on a direction to move in.
     * @return the direction to move in
     */
    public Direction decideMove() {
        return brain.decideMove();
    }

    /**
     * Sets the brain.
     * @param b the new brain
     */
    public void setBrain(Brain b) {
        brain = b;
        brain.setWorld(world);
    }

    /**
     * Returns the current brain.
     * @return the current brain
     */
    public Brain getBrain() {
        return brain;
    }

    /**
     * Returns the world in which the animal lives.
     * @return the world in which the animal lives
     */
    public World getWorld() {
        return world;
    }
}
