package dk.au.daimi.rabbithunt.model;

import dk.au.daimi.rabbithunt.model.world.Direction;
import dk.au.daimi.rabbithunt.model.world.World;

/**
 * The brain of a fox.
 * 
 * @author David Matuszek
 * @author Martin Geisler
 * @author Mikkel Krigrd
 * @version March 11, 2005
 */
public class FoxBrain extends Brain
{
	// instance variables
	private boolean haveSeenRabbit = false;
	private int distanceToRabbit;
	private Direction directionToRabbit;
	private Direction currentDirection;

	/**
	 * Creates a new fox brain.
	 * 
	 * @param w the world in which the fox lives
	 */
	public FoxBrain()
	{
		currentDirection = Direction.randomDirection();
	}
	/**
	 * Controls the movement of the fox.
	 * 
	 * @return the direction in which the fox wishes to move
	 */
	public Direction decideMove()
	{
		// look all around for rabbit
		for (Direction direction : Direction.DIRECTIONS)
		{
			if (world.look(direction) == World.RABBIT)
			{
				haveSeenRabbit = true;
				directionToRabbit = direction;
				distanceToRabbit = world.distance(direction);
			}
		}
		// if rabbit has been seen recently (not necessarily this time),
		// move toward its last known position
		if (haveSeenRabbit)
		{
			if (distanceToRabbit > 0)
			{
				distanceToRabbit--;
				return directionToRabbit;
			} else
			{ // rabbit was here--where did it go?
				haveSeenRabbit = false;
				currentDirection = Direction.randomDirection();
			}
		}
		// either haven't seen rabbit, or lost track of rabbit
		// continue with current direction, maybe dodging bushes
		if (world.canMove(currentDirection))
			return currentDirection;
		else if (world.canMove(Direction.turn(currentDirection, 1)))
			return Direction.turn(currentDirection, 1);
		else if (world.canMove(Direction.turn(currentDirection, -1)))
			return Direction.turn(currentDirection, -1);
		else
		{
			currentDirection = Direction.randomDirection();
			for (int i = 0; i < 8; i++)
			{
				if (world.canMove(currentDirection))
					return currentDirection;
				else
					currentDirection = Direction.turn(currentDirection, 1);
			}
		}
		// stuck! cannot move
		return Direction.NONE;
	}
}
